--
-- CombineWorkAddons
--
--
-- @author John Deere 6930
-- @date 14/03/2018
--

CombineWorkAddons = {};
CombineWorkAddons.TOTAL_HECTARE_COUNTER = 1;
CombineWorkAddons.CURRENT_HECTARE_COUNTER = 2;

function CombineWorkAddons.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Combine, specializations) and SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations) ;
end

function CombineWorkAddons:preLoad(savegame)	
end

function CombineWorkAddons:load(savegame)	
	self.addCutterArea = Utils.appendedFunction(self.addCutterArea, CombineWorkAddons.addCutterArea);


	self.updateHectareCounter = SpecializationUtil.callSpecializationsFunction("updateHectareCounter");
	self.getHectareCounter = CombineWorkAddons.getHectareCounter;
	self.addHectareCounter = SpecializationUtil.callSpecializationsFunction("addHectareCounter");
	self.addToHectareCounters = SpecializationUtil.callSpecializationsFunction("addToHectareCounters");
	
	self.fiveSecondSaveTime = 0;
	self.maxLiterPerHour = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons#maxLiterPerHour"), 100000);

	self.lastLiterPerAreaFiveSum = 0;
	self.literPerAreaSum = 0;
	self.lastLiterPerHour = 0;
	self.lastLiterPerHourBiggerZero = (self.lastLiterPerHour > 0);
	
	self.lastHectarePerAreaFiveSum = 0;
	self.hectarePerAreaSum = 0;
	self.lastHectarePerHour = 0;
	self.lastHectarePerHourBiggerZero = (self.lastHectarePerHour > 0);
	
	self.lastLiterPerHectare = 0;
	
	self.hectareCounters = {};
	for i=1, 2 do
		local entry = {};
		entry.counter = 0.0;
		if i == 1 then
			entry.type = CombineWorkAddons.TOTAL_HECTARE_COUNTER;
		else
			entry.type = CombineWorkAddons.CURRENT_HECTARE_COUNTER;
		end;
		table.insert(self.hectareCounters, entry);
	end;
	
	self.hud = RafftnixGUI:new(nil, false, false);
    
    local uiScale = g_gameSettings:getValue("uiScale");	
	local startX = g_currentMission.vehicleHudBg.x --0.805
	local startY = g_currentMission.vehicleHudBg.y + g_currentMission.vehicleHudBg.height --0.1875
	local size = g_currentMission.vehicleHudBg.width --0.195
    local width = size;
    local height = size/1.5;
	
	local hudImageFilename = getXMLString(self.xmlFile, "vehicle.combineWorkAddons.hud#filename");
    self.hudImage = self.hud:createImage(self.hud.baseElement, startX, startY, width, height, hudImageFilename);
	
	local brandLogoFilename = getXMLString(self.xmlFile, "vehicle.combineWorkAddons.brand#filename");
	local brandPosX = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.brand#posX"), 0.0);
	local brandPosY = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.brand#posY"), 0.0);	
	local brandWidth = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.brand#width"), 0.5);	
	local brandHeight = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.brand#height"), 0.5);
	brandPosX = startX+width*brandPosX;
	brandPosY = startY+height*brandPosY;	
	brandWidth = width*brandWidth;	
	brandHeight = height*brandHeight;	
    self.brandLogo = self.hud:createImage(self.hud.baseElement, brandPosX, brandPosY, brandWidth, brandHeight, brandLogoFilename);
	
	local hudBarFilename = getXMLString(self.xmlFile, "vehicle.combineWorkAddons.hudBar#filename");
	local hudBarPosX = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.hudBar#posX"), 0.0);
	local hudBarPosY = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.hudBar#posY"), 0.0);	
	self.hudBarWidth = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.hudBar#width"), 0.5);	
	self.hudBarHeight = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.hudBar#height"), 0.5);
	hudBarPosX = startX+width*hudBarPosX;
	hudBarPosY = startY+height*hudBarPosY;	
	self.hudBarWidth = width*self.hudBarWidth;	
	self.hudBarHeight = height*self.hudBarHeight;	
    self.hudBar = self.hud:createImage(self.hud.baseElement, hudBarPosX, hudBarPosY, self.hudBarWidth, self.hudBarHeight, hudBarFilename);

	local hectareCounterIconFilename = getXMLString(self.xmlFile, "vehicle.combineWorkAddons.hectareCounterIcon#filename");
	local hectareCounterIconPosX = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.hectareCounterIcon#posX"), 0.0);
	local hectareCounterIconPosY = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.hectareCounterIcon#posY"), 0.0);	
	local hectareCounterIconWidth = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.hectareCounterIcon#width"), 0.5);	
	local hectareCounterIconHeight = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.hectareCounterIcon#height"), 0.5);
	hectareCounterIconPosX = startX+width*hectareCounterIconPosX;
	hectareCounterIconPosY = startY+height*hectareCounterIconPosY;	
	hectareCounterIconWidth = width*hectareCounterIconWidth;	
	hectareCounterIconHeight = height*hectareCounterIconHeight;	
    self.hectareCounterIcon = self.hud:createImage(self.hud.baseElement, hectareCounterIconPosX, hectareCounterIconPosY, hectareCounterIconWidth, hectareCounterIconHeight, hectareCounterIconFilename);
		
	local quantimeterIconFilename = getXMLString(self.xmlFile, "vehicle.combineWorkAddons.quantimeterIcon#filename");
	local quantimeterIconPosX = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.quantimeterIcon#posX"), 0.0);
	local quantimeterIconPosY = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.quantimeterIcon#posY"), 0.0);	
	local quantimeterIconWidth = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.quantimeterIcon#width"), 0.5);	
	local quantimeterIconHeight = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.combineWorkAddons.quantimeterIcon#height"), 0.5);
	quantimeterIconPosX = startX+width*quantimeterIconPosX;
	quantimeterIconPosY = startY+height*quantimeterIconPosY;	
	quantimeterIconWidth = width*quantimeterIconWidth;	
	quantimeterIconHeight = height*quantimeterIconHeight;	
    self.quantimeterIcon = self.hud:createImage(self.hud.baseElement, quantimeterIconPosX, quantimeterIconPosY, quantimeterIconWidth, quantimeterIconHeight, quantimeterIconFilename);
	
	local border = 0.0025
	local textX = startX+border;
	local textWidth = width-2*border;
	local textHeight = height-2*border;
	local textY = startY+textHeight;
	local textSize = 0.015*uiScale;
	
	
    self.workLoadText = self.hud:createText(self.hud.baseElement, textX, startY+(textHeight/5), textWidth, 1, "", textSize, 1);
    self.workLoadText:setTextFocus("middle", "bottom");
	
    self.hectareCounterTotalText = self.hud:createText(self.hud.baseElement, textX, textY-textSize, textWidth, 1, string.format(g_i18n:getText("info_hectareCounters_total"), 0.0), textSize, 1);
    self.hectareCounterTotalText:setTextFocus("right", "bottom");
    self.hectareCounterCurrentText = self.hud:createText(self.hud.baseElement, textX, textY-textSize*2, textWidth, 1, string.format(g_i18n:getText("info_hectareCounters_current"), 0.0), textSize, 1);
    self.hectareCounterCurrentText:setTextFocus("right", "bottom");
	
	self.quantimeter1Text = self.hud:createText(self.hud.baseElement, textX, textY-textSize*3, textWidth, 1, string.format(g_i18n:getText("info_quantimeter1"), 0), textSize, 1);
    self.quantimeter1Text:setTextFocus("right", "bottom");	
	self.quantimeter2Text = self.hud:createText(self.hud.baseElement, textX, textY-textSize*4, textWidth, 1, string.format(g_i18n:getText("info_quantimeter2"), 0), textSize, 1);
    self.quantimeter2Text:setTextFocus("right", "bottom");	
	
	local strawModeOnStart = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.combine.chopperSwitch#strawModeOnStart"), true);
    if not strawModeOnStart then
		self:setIsStrawEnabled(false, true, true);
	end;
end;

function CombineWorkAddons:postLoad(savegame)
	local totalHectareCounter = nil;
	local currentHectareCounter = nil;
	if savegame ~= nil and not savegame.resetVehicles then
		totalHectareCounter = getXMLFloat(savegame.xmlFile, savegame.key.."#totalHectareCounter");
		currentHectareCounter = getXMLFloat(savegame.xmlFile, savegame.key.."#currentHectareCounter");
	end;
	if totalHectareCounter == nil then
		totalHectareCounter = 0;
	end;
	if currentHectareCounter == nil then
		currentHectareCounter = 0;
	end;
	self:updateHectareCounter(CombineWorkAddons.TOTAL_HECTARE_COUNTER, totalHectareCounter, true);
	self:updateHectareCounter(CombineWorkAddons.CURRENT_HECTARE_COUNTER, currentHectareCounter, true);	
end;

function CombineWorkAddons:getSaveAttributesAndNodes(nodeIdent)
	local totalHectareCounter = self:getHectareCounter(CombineWorkAddons.TOTAL_HECTARE_COUNTER);
	local currentHectareCounter = self:getHectareCounter(CombineWorkAddons.CURRENT_HECTARE_COUNTER);	
	local attributes = 'totalHectareCounter="'..totalHectareCounter..'" currentHectareCounter="'..currentHectareCounter..'"';
	local nodes = "";
	return attributes,nodes;
end;

function CombineWorkAddons:delete()
	if self.hud ~= nil then
		self.hud:delete();
	end;
end

function CombineWorkAddons:mouseEvent(posX, posY, isDown, isUp, button)
end

function CombineWorkAddons:keyEvent(unicode, sym, modifier, isDown)
end

function CombineWorkAddons:readStream(streamId, connection)
	for _,counter in pairs(self.hectareCounters) do
		local hectareCount = streamReadFloat32(streamId);
		self:updateHectareCounter(counter.type, hectareCount, true);
	end;
	
	self.lastLiterPerHourBiggerZero = streamReadBool(streamId);		
	if self.lastLiterPerHourBiggerZero then
		self.lastLiterPerHour = streamReadFloat32(streamId);
	end;		
	self.lastHectarePerHourBiggerZero = streamReadBool(streamId);		
	if self.lastHectarePerHourBiggerZero then
		self.lastHectarePerHour = streamReadFloat32(streamId);
	end;
end;

function CombineWorkAddons:writeStream(streamId, connection)
	for _,counter in pairs(self.hectareCounters) do
		local hectareCount = self:getHectareCounter(counter.type);
		streamWriteFloat32(streamId, hectareCount);
	end;
	
	streamWriteBool(streamId, self.lastLiterPerHourBiggerZero)
	if self.lastLiterPerHourBiggerZero then
		streamWriteFloat32(streamId, self.lastLiterPerHour);
	end;
	streamWriteBool(streamId, self.lastHectarePerHourBiggerZero)
	if self.lastHectarePerHourBiggerZero then
		streamWriteFloat32(streamId, self.lastHectarePerHour);
	end;
end;

function CombineWorkAddons:readUpdateStream(streamId, timestamp, connection)
    if connection:getIsServer() then	
        self.lastLiterPerHourBiggerZero = streamReadBool(streamId);		
        if self.lastLiterPerHourBiggerZero then
            self.lastLiterPerHour = streamReadFloat32(streamId);
        end;		
        self.lastHectarePerHourBiggerZero = streamReadBool(streamId);		
        if self.lastHectarePerHourBiggerZero then
            self.lastHectarePerHour = streamReadFloat32(streamId);
        end;
    end;
end;

function CombineWorkAddons:writeUpdateStream(streamId, connection, dirtyMask)
    if not connection:getIsServer() then
        streamWriteBool(streamId, self.lastLiterPerHourBiggerZero)
        if self.lastLiterPerHourBiggerZero then
			streamWriteFloat32(streamId, self.lastLiterPerHour);
        end;
        streamWriteBool(streamId, self.lastHectarePerHourBiggerZero)
        if self.lastHectarePerHourBiggerZero then
			streamWriteFloat32(streamId, self.lastHectarePerHour);
        end;
	end;
end;


function CombineWorkAddons:update(dt)

	if self:getIsActiveForInput() then
		local curHectareCounter = self:getHectareCounter(CombineWorkAddons.CURRENT_HECTARE_COUNTER);
		if curHectareCounter ~= nil and curHectareCounter > 0 then
			if InputBinding.hasEvent(InputBinding.RESET_HECTARECOUNTER) then
				self:updateHectareCounter(CombineWorkAddons.CURRENT_HECTARE_COUNTER, 0);
				g_currentMission:showBlinkingWarning(g_i18n:getText("warning_resetHectareCounter"), 2000);
			end;
		end;
        if InputBinding.hasEvent(InputBinding.HIDEGUI) then
            self.hud:setHideGUI(not self.hud.hideGUI);
        end;
	end;
end;

function CombineWorkAddons:updateTick(dt)
    if self.hud.baseElement.visibility ~= self:getIsActiveForInput(false) then
        if self:getIsActiveForInput(false) then
            self.hud:open();
        else
            self.hud:close();
        end;
    end;
end;

function CombineWorkAddons:addCutterArea(cutter, area, realArea, inputFruitType, outputFillType)
	if self.isServer then	
		self.lastLiterPerHourBiggerZero = false;
		self.lastHectarePerHourBiggerZero = false;
		self.lastLiterPerHour = 0;
		self.lastHectarePerHour = 0;

		local literPerArea = 0;	
		local hectare = 0;	
		
		if area ~= 0 then
			if FruitUtil.fruitIndexToDesc[inputFruitType] ~= nil then
				literPerArea = ((area * g_currentMission:getFruitPixelsToSqm())*FruitUtil.fruitIndexToDesc[inputFruitType].literPerSqm*self.threshingScale);		
			end;
			
			hectare = Utils.areaToHa(realArea, g_currentMission:getFruitPixelsToSqm()) -- 4096px are mapped to 2048m
			
			self:addToHectareCounters(hectare);
		end;
			
		self.literPerAreaSum = self.literPerAreaSum + literPerArea;
		self.hectarePerAreaSum = self.hectarePerAreaSum + hectare;
		if self.fiveSecondSaveTime <= g_currentMission.time then
			self.lastLiterPerAreaFiveSum = self.literPerAreaSum;
			self.lastHectarePerAreaFiveSum = self.hectarePerAreaSum;
			self.literPerAreaSum = 0;
			self.hectarePerAreaSum = 0;
			self.fiveSecondSaveTime = g_currentMission.time+1000;
		end;
		
		self.lastLiterPerHour = self.lastLiterPerAreaFiveSum*60*60;
		self.lastHectarePerHour = self.lastHectarePerAreaFiveSum*60*60;
		
		self.lastLiterPerHourBiggerZero = (self.lastLiterPerHour > 0);
		self.lastHectarePerHourBiggerZero = (self.lastHectarePerHour > 0);
	end;
	self.lastLiterPerHectare = self.lastLiterPerHour/math.max(self.lastHectarePerHour, 0.1);
		
end;


function CombineWorkAddons:draw()
	if self:getIsActiveForInput(false) then
        if self.hud.hideGUI then
            g_currentMission:addHelpButtonText(g_i18n:getText("showGUI"), InputBinding.HIDEGUI); 
        else
            g_currentMission:addHelpButtonText(g_i18n:getText("hideGUI"), InputBinding.HIDEGUI); 
        end;
		
		local percentage = math.min(self.lastLiterPerHour/self.maxLiterPerHour, 1);		
		self.workLoadText:setText(g_i18n:getText("info_workload")..string.format(" %5.0f %%", 100*percentage));
		
		--print("lastLiter/Max Liter: "..self.lastLiterPerHour.." / "..self.maxLiterPerHour)
		
		local width = self.hudBarWidth*percentage;
		self.hudBar:setDimension(width, self.hudBarHeight)
		
		local curHectareCounter = Utils.getNoNil(self:getHectareCounter(CombineWorkAddons.CURRENT_HECTARE_COUNTER), 0.0);
		local totalHectareCounter = Utils.getNoNil(self:getHectareCounter(CombineWorkAddons.TOTAL_HECTARE_COUNTER), 0.0);		
		self.hectareCounterTotalText:setText(string.format(g_i18n:getText("info_hectareCounters_total"), totalHectareCounter));
		self.hectareCounterCurrentText:setText(string.format(g_i18n:getText("info_hectareCounters_current"), curHectareCounter));
		
		if curHectareCounter > 0 then
			g_currentMission:addHelpButtonText(g_i18n:getText("action_resetHectareCounter"), InputBinding.RESET_HECTARECOUNTER, nil, GS_PRIO_HIGH);
		end;
		
		self.quantimeter1Text:setText(string.format(g_i18n:getText("info_quantimeter1"), self.lastHectarePerHour));
		self.quantimeter2Text:setText(string.format(g_i18n:getText("info_quantimeter2"), self.lastLiterPerHectare));
	end;	
end;


function CombineWorkAddons:updateHectareCounter(hectareCounterType, hectareCount, noEventSend)
	local hectareCounter = self.hectareCounters[hectareCounterType];
    if hectareCounter ~= nil then
		UpdateHectareCounterEvent.sendEvent(self, hectareCounterType, hectareCount, noEventSend);
		hectareCounter.counter = hectareCount;
	end;
end;

function CombineWorkAddons:getHectareCounter(hectareCounterType)
	local hectareCounter = self.hectareCounters[hectareCounterType];
    if hectareCounter ~= nil then
		return hectareCounter.counter;
	end;
end;

function CombineWorkAddons:addHectareCounter(hectareCounterType, addValue)
    if addValue ~= nil and addValue~= 0 then
		local curCounter = self:getHectareCounter(hectareCounterType);
		curCounter = curCounter+addValue;
		if curCounter > 9999.99 then
			curCounter = addValue;
		end;
		self:updateHectareCounter(hectareCounterType, curCounter);
	end;
end;

function CombineWorkAddons:addToHectareCounters(addValue)
    if addValue ~= nil and addValue~= 0 then
		for _,counter in pairs(self.hectareCounters) do
			self:addHectareCounter(counter.type, addValue);
		end;
	end;
end;
